%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Technical Note 170
% Parameters for 'Virtual synchronous generator' system
% Copyright imperix ltd, 2024
% dev@imperix.ch
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clearvars -except SIMULINK_MODE variant_generation variant_simulation

%% SYSTEM PARAMETERS
SwitchingFreq = 50e3;           % Switching frequency [Hz]
ControlFreq = SwitchingFreq;    % Control frequency [Hz]
ControlPeriod = 1/ControlFreq;  % Control period [s]
SamplingPhase = 0.5;            % Sampling phase [-]

Vdc0 = 800;                     % Initial DC voltage [V]
Vg = 230;                       % RMS Grid voltage [V]
Freq = 50;                      % Grid frequency [Hz]
Lg = 0.95e-3;                   % TPI main inductor value [H]
Rg = 54e-3;                     % Series equivalent resistance of Lg [ohm]
Cf = 12.9e-6;                   % Capacitance of the filter [F]

%% GRID CURRENT CONTROLLER PARAMETERS
% Optimum magnitude criterion

K1 = 1/Rg;
T1 = Lg/Rg;
Td1 = (1/2 + 1/2 + 1/2)*ControlPeriod;    % Total delay (control delay Ts/2, modulator delay Ts/2, synchronous averaging delay Ts/2)
Tn1 = T1;
Ti1 = 2*K1*Td1;
Kp_I = Tn1/Ti1;                         % Proportional gain for the GFMI current controller
Ki_I = 1/Ti1;                           % Integral gain for the GFMI current controller

%% GRID VOLTAGE CONTROLLER PARAMETERS
% Symmetrical optimum criterion

T2 = Cf;
Tdeq = 10*Td1;                          % Voltage controller 10 time slower than the GFMI current controller for accurate voltage tracking
a = 2;                                  % change the pole placement of the control function (usually between 2 and 4)
Ti2 = a^2*Tdeq;
Kp_V = T2/(a*Tdeq);                     % Proportional gain for the GFMI voltage controller
Ki_V = Kp_V/Ti2;                        % Integral gain for the GFMI voltage controller

%% VIRTUAL SINCHRONOUS GENERATOR CONTROLLER PARAMETERS

w0 = 2*pi*Freq;                         % Nominal frequency [rad/s]
delta_w_pu=1/100;                       % Maximum frequency deviation (%)
delta_w=delta_w_pu*w0;                  % Maximum frequency deviation [rad/s]
P_max=18e3;                             % Maximum active power [W]
m=delta_w/P_max;                        % Frequency static droop coefficient [rads^-1/W] 
S_max=22e3;                             % Apparent power [VA]
delta_V_pu=10/100;                      % Maximum voltage deviation (%)
delta_V=230*sqrt(2)*delta_V_pu;         % Maximum voltage deviation [V]
Q_max=sqrt(S_max^2-P_max^2);            % Reactive power [var]
n=delta_V/Q_max;                        % Voltage static droop coefficient [V/var]

ro=2*pi;                                % Maximum allowed RoCoF [rad/s^2]
tau_J=m*P_max/ro;                       % Inertial time constant [s]
J=tau_J/(m*w0);                         % Inertia [kgm^2]